//Software base para OPS012
//Controlador ESP32

//Biblioteca para temperatura/carga
//É preciso instalar a biblioteca em "Gerenciar Bibliotecas..."
#include <HX711_ADC.h>

//Definição dos pinos do controlador
//Saidas digiais
#define pin_Y1 13
#define pin_Y2 12
#define pin_Y3 27
#define pin_Y4 26
#define pin_Y5 25
#define pin_Y6 33

//Entradas digitais
#define pin_X1 15
#define pin_X2 14
#define pin_X3 39
#define pin_X4 36

//Entradas analógicas
#define pin_AI1 35
#define pin_AI2 34

//Saidas analogicas
#define pin_AO1 32
#define pin_AO2 23

//Pinos auxiliares
#define ENTX_pin 2 //Pino de comunicação RS485

//Entradas de temperatura / célula de carga
const int pin_DOUT1 = 18;
const int pin_SCK1 = 5;
const int pin_DOUT2 = 19;
const int pin_SCK2 = 4;

//Variaveis para controle das entradas e saidas
bool Digital_Out[6];
bool Digital_In[4];
int Analog_In[2];
int Analog_Out[2];
long HxADC_In[2];

//chamada da biblioteca de leitura do conversor ADC
HX711_ADC Sensor1(pin_DOUT1, pin_SCK1);
HX711_ADC Sensor2(pin_DOUT2, pin_SCK2);

void setup() {
  //Inicializa comunicações
  Serial.begin(115200); // (USB)
  Serial1.begin(115200); // (TTL)
  Serial2.begin(115200); //(485)

  //Inicializa hardware
  pinMode(pin_X1, INPUT);
  pinMode(pin_X2, INPUT);
  pinMode(pin_X3, INPUT);
  //A leitura do X4 é feita como analógica

  pinMode(pin_Y1, OUTPUT);
  pinMode(pin_Y2, OUTPUT);
  pinMode(pin_Y3, OUTPUT);
  pinMode(pin_Y4, OUTPUT);
  pinMode(pin_Y5, OUTPUT);
  pinMode(pin_Y6, OUTPUT);

  digitalWrite(pin_Y1, LOW);
  digitalWrite(pin_Y2, LOW);
  digitalWrite(pin_Y3, LOW);
  digitalWrite(pin_Y4, LOW);
  digitalWrite(pin_Y5, LOW);
  digitalWrite(pin_Y6, LOW);

  // Configuração da saída analógica
  //(Canal, Frequência, Resolução)
  ledcSetup(0, 1000, 12);
  ledcSetup(1, 1000, 12);
  //(Pino, Canal)
  ledcAttachPin(pin_AO1, 0);
  ledcAttachPin(pin_AO2, 1);
  
  //Inicialização da biblioteca HX711
  Sensor1.begin();
  Sensor1.start(2000, false);
  Sensor1.setCalFactor(1);

  Sensor2.begin();
  Sensor2.start(2000, false);
  Sensor2.setCalFactor(1);
}

void loop() {
  //************************************************************************************
  //Leitura das entradas digitais
  //Esse código deve rodar sempre para leitura das entradas digitais
  //Utilizar o Array Digital_In[0] a Digital_In[3] no software para controle
  //************************************************************************************
  Digital_In[0] = !digitalRead(pin_X1);
  Digital_In[1] = !digitalRead(pin_X2);
  Digital_In[2] = !digitalRead(pin_X3);
  if (analogRead(pin_X4) > 100){
    Digital_In[3] = 0;
  }else{
    Digital_In[3] = 1;
  }

  //************************************************************************************
  //Escrita das saídas digitais
  //Esse código deve rodar sempre para escrita das saidas digitais
  //Utilizar o Array Digital_Out[0] a Digital_Out[5] no software para controle
  //************************************************************************************
  digitalWrite(pin_Y1, Digital_Out[0]);
  digitalWrite(pin_Y2, Digital_Out[1]);
  digitalWrite(pin_Y3, Digital_Out[2]);
  digitalWrite(pin_Y4, Digital_Out[3]);
  digitalWrite(pin_Y5, Digital_Out[4]);
  digitalWrite(pin_Y6, Digital_Out[5]);

  //************************************************************************************
  //Leitura das entradas Analógicas
  //Esse código deve rodar sempre para leitura das entradas analógicas
  //Utilizar o Array Analog_In[0] a Analog_In[1] no software para controle
  //************************************************************************************
  Analog_In[0] = analogRead(pin_AI1);
  Analog_In[1] = analogRead(pin_AI2);

  //************************************************************************************
  //Escrita das saídas Analógicas / 0 >= dutyCycle <= 3722 (0-10V)
  //Esse código deve rodar sempre para escrita das saidas analógicas
  //Utilizar o Array Analog_Out[0] a Analog_Out[1] no software para controle
  //************************************************************************************
  ledcWrite(0, Analog_Out[0]);
  ledcWrite(1, Analog_Out[0]);

  //************************************************************************************
  //Leitura das entradas dos conversores ADC
  //Esse código deve rodar sempre para leitura das entradas ADC
  //Utilizar o Array HxADC_In[0] a HxADC_In[1] no software para controle
  //O valor de entrada tem 24 bits de resolução
  //************************************************************************************
  if (Sensor1.update()) {
    HxADC_In[0] = Sensor1.getData();
  }

  if (Sensor2.update()) {
    HxADC_In[1] = Sensor2.getData();
  }

}
